@@ -4,7 +4,7 @@
 ##
 ##    class "fv" of function value objects
 ##
-##    $Revision: 1.177 $   $Date: 2022/08/11 05:00:35 $
+##    $Revision: 1.181 $   $Date: 2023/09/08 07:32:16 $
 ##
 ##
 ##    An "fv" object represents one or more related functions
@@ -821,8 +821,8 @@ rebadge.fv <- function(x, new.ylab, new.fname,
 
 ## common invocations to label a function like Kdot or Kcross
 rebadge.as.crossfun <- function(x, main, sub=NULL, i, j) {
-  i <- make.parseable(i)
-  j <- make.parseable(j)
+  i <- make.parseable(paste(i))
+  j <- make.parseable(paste(j))
   if(is.null(sub)) {
     ## single function name like "K"
     ylab <- substitute(main[i, j](r),
@@ -845,7 +845,7 @@ rebadge.as.crossfun <- function(x, main, sub=NULL, i, j) {
 }
 
 rebadge.as.dotfun <- function(x, main, sub=NULL, i) {
-  i <- make.parseable(i)
+  i <- make.parseable(paste(i))
   if(is.null(sub)) {
     ## single function name like "K"
     ylab <- substitute(main[i ~ dot](r),
@@ -1156,35 +1156,29 @@ with.fv <- function(data, expr, ..., fun=NULL, enclos=NULL) {
   return(out)
 }
 
-## method for 'range'
-
-range.fv <- local({
-
-  getValues <- function(x) {
-    xdat <- as.matrix(as.data.frame(x))
-    yall <- fvnames(x, ".")
-    vals <- xdat[, yall]
-    return(as.vector(vals))
+## integral of fv object
+integral.fv <- function(f, domain=NULL, ...) {
+  verifyclass(f, "fv")
+  df <- as.data.frame(f)
+  xname <- fvnames(f, ".x")
+  x <- df[,xname]
+  if(!is.null(domain)) {
+    check.range(domain)
+    xr <- range(x)
+    if(!all(inside.range(domain, xr)))
+      warning(paste("domain of integration", prange(domain),
+                    "was clipped to the available range of function values",
+                    prange(xr)), call.=FALSE)
+    ok <- inside.range(x, domain)
+    df <- df[ok, , drop=FALSE]
+    x <- x[ok]
   }
-  
-  range.fv <- function(..., na.rm=TRUE, finite=na.rm) {
-    aarg <- list(...)
-    isfun <- sapply(aarg, is.fv)
-    if(any(isfun)) 
-      aarg[isfun] <- lapply(aarg[isfun], getValues)
-    z <- do.call(range, append(aarg, list(na.rm=na.rm, finite=finite)))
-    return(z)
-  }
-
-  range.fv
-})
-
-min.fv <- function(..., na.rm=TRUE, finite=na.rm) {
-  range(..., na.rm=TRUE, finite=na.rm)[1L]
-}
-
-max.fv <- function(..., na.rm=TRUE, finite=na.rm) {
-  range(..., na.rm=TRUE, finite=na.rm)[2L]
+  integrands <- as.matrix(df[, colnames(df) != xname, drop=FALSE])
+  dx <- diff(x)
+  #' trapezoidal rule
+  wx <- (c(dx, 0) + c(0, dx))/2
+  ans <- wx %*% integrands
+  return(ans[1,])
 }
 
   
